/*
 * Copyright (c) 2017-2018 The Linux Foundation. All rights reserved.
 *
 * Permission to use, copy, modify, and/or distribute this software for
 * any purpose with or without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
 * AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

#ifndef _WLAN_CRYPTO_FILS_API_H_
#define _WLAN_CRYPTO_FILS_API_H_

#ifdef WLAN_CRYPTO_FILS_OS_DERIVATIVE
static inline const struct wlan_crypto_cipher *fils_register(void)
{
	return NULL;
}
#else
/**
 * fils_register() - Register all callback functions to Crypto manager
 *
 * This function is invoked from crypto object manager to register
 * FILS specific callbacks.
 *
 * Return: Pointer to wlan_crypto_cipher Object
 */
const struct wlan_crypto_cipher *fils_register(void);
#endif

#ifdef WLAN_SUPPORT_FILS
/**
 * wlan_crypto_fils_delkey - Delete fils aad key
 * @peer: Peer object
 *
 * This function delete the peer specific FILS AAD key data.
 *
 * Return: None
 */
void wlan_crypto_fils_delkey(struct wlan_objmgr_peer *peer);
#endif /* WLAN_SUPPORT_FILS */

#endif /* End of _WLAN_CRYPTO_FILS_API_H_ */

